"""
Credits: https://github.com/mkaz/termgraph
Had a small issue with colors while using no_values 
and moreover the repo is maintained 
so copied the codes instead of using it directly
"""
from . import *